from collections import OrderedDict
from huawei_lte_api.ApiGroup import ApiGroup


class Monitoring(ApiGroup):
    def converged_status(self) -> dict:
        return self._connection.get('monitoring/converged-status')

    def status(self) -> dict:
        return self._connection.get('monitoring/status')

    def check_notifications(self) -> dict:
        return self._connection.get('monitoring/check-notifications')

    def traffic_statistics(self) -> dict:
        return self._connection.get('monitoring/traffic-statistics')

    def start_date(self) -> dict:
        return self._connection.get('monitoring/start_date')

    def set_start_date(self, start_day: int, data_limit: str, month_threshold: int):
        """
        Sets network usage alarm for LTE
        :param start_day: number of day when monitoring starts
        :param data_limit: Maximal data limit as string eg.: 1000MB or 1GB and so on
        :param month_threshold: Alarm threshold in % as int number eg.: 90
        :return: dict
        """
        return self._connection.post('monitoring/start_date', OrderedDict((
            ('StartDay', start_day),
            ('DataLimit', data_limit),
            ('MonthThreshold', month_threshold),
            ('SetMonthData', 1)
        )))

    def start_date_wlan(self) -> dict:
        return self._connection.get('monitoring/start_date_wlan')

    def set_start_date_wlan(self, start_day: int, data_limit: str, month_threshold: int):
        """
        Sets network usage alarm for WLAN
        :param start_day: number of day when monitoring starts
        :param data_limit: Maximal data limit as string eg.: 1000MB or 1GB and so on
        :param month_threshold: Alarm threshold in % as int number eg.: 90
        :return: dict
        """
        return self._connection.post('monitoring/start_date_wlan', OrderedDict((
            ('StartDay', start_day),
            ('DataLimit', data_limit),
            ('MonthThreshold', month_threshold),
            ('SettingEnable', 1)  #!FIXME
        )))

    def month_statistics(self) -> dict:
        return self._connection.get('monitoring/month_statistics')

    def month_statistics_wlan(self) -> dict:
        return self._connection.get('monitoring/month_statistics_wlan')

    def set_clear_traffic(self) -> dict:
        return self._connection.post('monitoring/clear-traffic', {
            'ClearTraffic': 1
        })




from huawei_lte_api.ApiGroup import ApiGroup


class FileManager(ApiGroup):
    def upload(self, uploadfile: str):
        return self._connection.post('filemanager/upload', {
            'uploadfile': uploadfile
        })  # !FIXME see issue #2



from collections import OrderedDict
from huawei_lte_api.ApiGroup import ApiGroup
from huawei_lte_api.AuthorizedConnection import authorized_call


class Pb(ApiGroup):
    @authorized_call
    def get_pb_match(self, phone_number: str) -> dict:
        """
    Find number in PhoneBook
    :param phone_number:
    :return:
    """
        return self._connection.post('pb/pb-match', {
            'Phone': phone_number
        })

    @authorized_call
    def get_pb_list(self,
                    page: int = 1,
                    key_word: str='',
                    group_id: int=0,
                    read_count: int=50
                    ) -> dict:
        return self._connection.post('pb/pb-list', OrderedDict((
            ('GroupID', group_id),
            ('PageIndex', page),
            ('ReadCount', read_count),
            ('KeyWord', key_word)
        )))



import datetime
from collections import OrderedDict
from huawei_lte_api.ApiGroup import ApiGroup


class Host(ApiGroup):
    def info(self, date_time: datetime.datetime, platform: str, user_agent: str, version: str):
        return self._connection.post('host/info', OrderedDict((
            ('Time', date_time.strftime('%Y%m%d%H%M%S')),
            ('Timezone', 'GMT{}'.format(date_time.strftime('%z'))),
            ('Platform', platform),
            ('PlatformVer', user_agent),
            ('Navigator', version),
            ('NavigatorVer', user_agent)
        )))



import base64
import hashlib
from huawei_lte_api.enums.user import PasswordTypeEnum, LoginStateEnum, LoginErrorEnum
from huawei_lte_api.enums.client import ResponseEnum
from huawei_lte_api.ApiGroup import ApiGroup
from huawei_lte_api.AuthorizedConnection import authorized_call
from huawei_lte_api.exceptions import ResponseErrorException, \
    LoginErrorAlreadyLoginException, \
    LoginErrorUsernamePasswordModifyException, \
    LoginErrorUsernamePasswordOverrunException, \
    LoginErrorUsernamePasswordWrongException, \
    LoginErrorUsernameWrongException, \
    LoginErrorPasswordWrongException


class User(ApiGroup):
    _username = 'admin'
    _password = None

    def __init__(self, connection, username: str=None, password: str=None):
        super(User, self).__init__(connection)
        self._username = username if username else 'admin'
        self._password = password

    def state_login(self) -> dict:
        return self._connection.get('user/state-login')

    def _login(self, password_type: PasswordTypeEnum=PasswordTypeEnum.BASE_64) -> bool:
        if not self._password:
            password = b''
        else:
            if password_type == PasswordTypeEnum.SHA256:
                concentrated = b''.join([
                    self._username.encode('UTF-8'),
                    base64.b64encode(hashlib.sha256(self._password.encode('UTF-8')).hexdigest().encode('ascii')),
                    self._connection.request_verification_tokens[0].encode('UTF-8')
                ])
                password = base64.b64encode(hashlib.sha256(concentrated).hexdigest().encode('ascii'))
            else:
                password = base64.b64encode(self._password.encode('UTF-8'))

        try:
            result = self._connection.post('user/login', {
                'Username': self._username,
                'Password': password.decode('UTF-8'),
                'password_type': password_type.value
            }, refresh_csfr=True)
        except ResponseErrorException as e:
            error_code_to_message = {
                LoginErrorEnum.USERNAME_WRONG: 'Username wrong',
                LoginErrorEnum.PASSWORD_WRONG: 'Password wrong',
                LoginErrorEnum.ALREADY_LOGIN: 'Already login',
                LoginErrorEnum.USERNAME_PWD_WRONG: 'Username and Password wrong',
                LoginErrorEnum.USERNAME_PWD_ORERRUN: 'Password overrun',
                LoginErrorEnum.USERNAME_PWD_MODIFY: 'Password modify',
            }

            error_code_to_exception = {
                LoginErrorEnum.USERNAME_WRONG: LoginErrorUsernameWrongException,
                LoginErrorEnum.PASSWORD_WRONG: LoginErrorPasswordWrongException,
                LoginErrorEnum.ALREADY_LOGIN: LoginErrorAlreadyLoginException,
                LoginErrorEnum.USERNAME_PWD_WRONG: LoginErrorUsernamePasswordWrongException,
                LoginErrorEnum.USERNAME_PWD_ORERRUN: LoginErrorUsernamePasswordOverrunException,
                LoginErrorEnum.USERNAME_PWD_MODIFY: LoginErrorUsernamePasswordModifyException,
            }

            message = error_code_to_message.get(e.code, 'Unknown')
            raise error_code_to_exception.get(e.code, ResponseErrorException)(
                '{}: {}'.format(e.code, message), e.code)

        return result == ResponseEnum.OK.value

    def login(self, force_new_login: bool=False) -> bool:
        state_login = self.state_login()
        if LoginStateEnum(int(state_login['State'])) == LoginStateEnum.LOGGED_IN and not force_new_login:
            return True

        return self._login(PasswordTypeEnum(int(state_login['password_type'])))

    @authorized_call
    def logout(self):
        return self._connection.post('user/logout', {
            'Logout': 1
        })

    def remind(self) -> dict:
        return self._connection.get('user/remind')

    def password(self) -> dict:
        return self._connection.get('user/password')

    def pwd(self) -> dict:
        return self._connection.get('user/pwd')

    def set_remind(self, remind_state):
        return self._connection.post('user/remind', {
            'remindstate': remind_state
        })



from huawei_lte_api.ApiGroup import ApiGroup
from huawei_lte_api.AuthorizedConnection import authorized_call


class SNtp(ApiGroup):
    def get_settings(self) -> dict:
        return self._connection.get('sntp/settings')

    @authorized_call
    def sntpswitch(self) -> dict:
        return self._connection.get('sntp/sntpswitch')

    def serverinfo(self) -> dict:
        return self._connection.get('sntp/serverinfo')

    def timeinfo(self) -> dict:
        return self._connection.get('sntp/timeinfo')



from huawei_lte_api.ApiGroup import ApiGroup


class DeviceInformation(ApiGroup):
    def config(self):
        return self._connection.get(
            'deviceinformation/config.xml', prefix='config')



import datetime
from collections import OrderedDict
from huawei_lte_api.ApiGroup import ApiGroup
from huawei_lte_api.AuthorizedConnection import authorized_call
from huawei_lte_api.enums.sms import BoxTypeEnum, TextModeEnum, SaveModeEnum, SendTypeEnum, PriorityEnum


class Sms(ApiGroup):
    def get_cbsnewslist(self) -> dict:
        return self._connection.get('sms/get-cbsnewslist')

    def sms_count(self) -> dict:
        return self._connection.get('sms/sms-count')

    def splitinfo_sms(self) -> dict:
        return self._connection.get('sms/splitinfo-sms')

    @authorized_call
    def sms_feature_switch(self) -> dict:
        return self._connection.get('sms/sms-feature-switch')

    def send_status(self) -> dict:
        return self._connection.get('sms/send-status')

    @authorized_call
    def get_sms_list(self,
                     page: int=1,
                     box_type: BoxTypeEnum=BoxTypeEnum.LOCAL_INBOX,
                     read_count: int=20,
                     sort_type: int=0,
                     ascending: int=0,
                     unread_preferred: int=0
                     ) -> dict:
        # Note: at least the B525s-23a is order sensitive
        return self._connection.post('sms/sms-list', OrderedDict((
            ('PageIndex', page),
            ('ReadCount', read_count),
            ('BoxType', box_type.value),
            ('SortType', sort_type),
            ('Ascending', ascending),
            ('UnreadPreferred', unread_preferred),
        )))

    @authorized_call
    def delete_sms(self, sms_ids: list):
        data = []
        for sms_id in sms_ids:
            data.append({'Index': sms_id})
        return self._connection.post('sms/delete-sms', data)

    @authorized_call
    def backup_sim(self, from_date: datetime.datetime, is_move: bool=False):
        return self._connection.post('sms/backup-sim', OrderedDict((
            ('IsMove', int(is_move)),
            ('Date', from_date.strftime("%Y-%m-%d %H:%M:%S"))
        )))

    @authorized_call
    def set_read(self, sms_id: int):
        return self._connection.post('sms/set-read', {
            'Index': sms_id
        })

    @authorized_call
    def save_sms(self,
                 phone_numbers: list,
                 message: str,
                 sms_index: int=-1,
                 sca: str='',
                 text_mode: TextModeEnum=TextModeEnum.SEVEN_BIT,
                 from_date: datetime.datetime=None,
                 ):

        if from_date is None:
            from_date = datetime.datetime.utcnow()
        dicttoxml_xargs = {
            'item_func': lambda x: x[:-1]
        }

        return self._connection.post('sms/save-sms', OrderedDict((
            ('Index', sms_index),
            ('Phones',  phone_numbers),
            ('Sca', sca),
            ('Content', message),
            ('Length', len(message)),
            ('Reserved', text_mode.value),
            ('Date', from_date.strftime("%Y-%m-%d %H:%M:%S"))
        )), dicttoxml_xargs=dicttoxml_xargs)

    @authorized_call
    def send_sms(self,
                 phone_numbers: list,
                 message: str,
                 sms_index: int=-1,
                 sca: str='',
                 text_mode: TextModeEnum=TextModeEnum.SEVEN_BIT,
                 from_date: datetime.datetime=None,
                 ):

        if from_date is None:
            from_date = datetime.datetime.utcnow()
        dicttoxml_xargs = {
            'item_func': lambda x: x[:-1]
        }

        return self._connection.post('sms/send-sms', OrderedDict((
            ('Index', sms_index),
            ('Phones', phone_numbers),
            ('Sca', sca),
            ('Content', message),
            ('Length', len(message)),
            ('Reserved', text_mode.value),
            ('Date', from_date.strftime("%Y-%m-%d %H:%M:%S"))
        )), dicttoxml_xargs=dicttoxml_xargs)

    def cancel_send(self):
        return self._connection.post('sms/cancel-send', {
            'request': 1,
        }, dicttoxml_xargs={
            'root': False,
        })

    @authorized_call
    def config(self) -> dict:
        return self._connection.get('sms/config')

    @authorized_call
    def set_config(self,
                   sca: str,
                   save_mode: SaveModeEnum=SaveModeEnum.LOCAL,
                   validity: int=10752,
                   use_s_report: bool=False,
                   send_type: SendTypeEnum=SendTypeEnum.SEND,
                   priority: PriorityEnum=PriorityEnum.NORMAL
                   ):
        return self._connection.post('sms/config', OrderedDict((
            ('SaveMode', save_mode.value),
            ('Validity', validity),
            ('Sca', sca),
            ('UseSReport', use_s_report),
            ('SendType', send_type.value),
            ('Priority', priority.value)
        )))



from huawei_lte_api.ApiGroup import ApiGroup


class UsbStorage(ApiGroup):

    def fsstatus(self) -> dict:
        return self._connection.get('usbstorage/fsstatus')

    def usbaccount(self) -> dict:
        return self._connection.get('usbstorage/usbaccount')



from collections import OrderedDict
from huawei_lte_api.ApiGroup import ApiGroup
from huawei_lte_api.AuthorizedConnection import authorized_call


class SdCard(ApiGroup):

    def dlna_setting(self) -> dict:
        return self._connection.get('sdcard/dlna-setting')

    @authorized_call
    def set_dlna_setting(self, enabled: bool, share_all: bool,
                         share_path: str = '/'):
        return self._connection.post('sdcard/dlna-setting', {
            'enabled': int(enabled),
            'sharepath': share_path,
            'shareallpath': int(share_all),
        })

    def sdcard(self) -> dict:
        return self._connection.get('sdcard/sdcard')

    def sdcardsamba(self) -> dict:
        return self._connection.get('sdcard/sdcardsamba')

    @authorized_call
    def set_sdcardsamba(self, enabled: bool,
                        server_name: str = 'homerouter.cpe',
                        server_description: str = 'samba server',
                        workgroup_name: str = 'WORKGROUP',
                        anonymous_access: bool = False,
                        printer_enabled: bool = True):
        return self._connection.post('sdcard/sdcardsamba', OrderedDict((
            ('enabled', int(enabled)),
            ('servername', server_name),
            ('serverdescription', server_description),
            ('workgroupname', workgroup_name),
            ('anonymousaccess', int(anonymous_access)),
            ('printerenable', int(printer_enabled)),
        )))

    def printerlist(self) -> dict:
        return self._connection.get('sdcard/printerlist')

    @authorized_call
    def share_account(self) -> dict:
        return self._connection.get('sdcard/share-account')




from huawei_lte_api.ApiGroup import ApiGroup
from huawei_lte_api.enums.device import AntennaTypeEnum
from huawei_lte_api.AuthorizedConnection import authorized_call


class Device(ApiGroup):
    @authorized_call
    def information(self) ->dict:
        return self._connection.get('device/information')

    def autorun_version(self) ->dict:
        return self._connection.get('device/autorun-version')

    def device_feature_switch(self) ->dict:
        return self._connection.get('device/device-feature-switch')

    def basic_information(self) ->dict:
        return self._connection.get('device/basic_information')

    def usb_tethering_switch(self) ->dict:
        return self._connection.get('device/usb-tethering-switch')

    def boot_time(self) ->dict:
        return self._connection.get('device/boot_time')

    def set_control(self, control: int=4):
        return self._connection.post('device/control', {
            'Control': control
        })

    def signal(self) ->dict:
        return self._connection.get('device/signal')

    @authorized_call
    def control(self, control: int):
        return self._connection.post('device/control', {
            'Control': control
        })

    def reboot(self) -> dict:
        return self.control(1)

    def antenna_status(self) -> dict:
        return self._connection.get('device/antenna_status')

    def get_antenna_settings(self) -> dict:
        return self._connection.get('device/antenna_settings')

    def set_antenna_settings(self, antenna_type: AntennaTypeEnum=AntennaTypeEnum.AUTO) -> dict:
        return self._connection.post('device/antenna_settings', {
            'antenna_type': antenna_type.value
        })

    def antenna_type(self) -> dict:
        return self._connection.get('device/antenna_type')



import enum


@enum.unique
class BoxTypeEnum(enum.IntEnum):
    LOCAL_INBOX = 1
    LOCAL_SENT = 2
    LOCAL_DRAFT = 3
    LOCAL_TRASH = 4
    SIM_INBOX = 5
    SIM_SENT = 6
    SIM_DRAFT = 7
    MIX_INBOX = 8
    MIX_SENT = 9
    MIX_DRAFT = 10


@enum.unique
class TextModeEnum(enum.IntEnum):
    UCS2 = 0
    SEVEN_BIT = 1
    EIGHT_BIT = 2


@enum.unique
class SaveModeEnum(enum.IntEnum):
    LOCAL = 0
    SIM_CARD = 1
    SIM_CARD_FIRST = 2
    LOCAL_FIRST = 3


@enum.unique
class SendTypeEnum(enum.IntEnum):
    SEND = 0
    SEND_AND_SAVE = 1


@enum.unique
class PriorityEnum(enum.IntEnum):
    NORMAL = 0
    INTERACTIVE = 1
    URGENT = 2
    EMERGENCY = 3




from huawei_lte_api.ApiGroup import ApiGroup


class WebServer(ApiGroup):
    def publickey(self) -> dict:
        return self._connection.get('webserver/publickey')

    def token(self) -> dict:
        return self._connection.get('webserver/token')

    def white_list_switch(self) -> dict:
        return self._connection.get('webserver/white_list_switch')




from huawei_lte_api.ApiGroup import ApiGroup


class Diagnosis(ApiGroup):
    def trace_route_result(self) -> dict:
        return self._connection.get('diagnosis/tracerouteresult')

    def diagnose_ping(self) -> dict:
        return self._connection.get('diagnosis/diagnose_ping')

    def set_diagnose_ping(self, host: str, timeout: int = 4000):
        return self._connection.post('diagnosis/diagnose_ping', dict(
            Host=host,
            Timeout=timeout,
        ))

    def diagnose_traceroute(self) -> dict:
        return self._connection.get('diagnosis/diagnose_traceroute')

    def set_diagnose_traceroute(self, host: str, timeout: int = 4000, max_hop_count: int = 30):
        return self._connection.post('diagnosis/diagnose_ping', dict(
            Host=host,
            MaxHopCount=max_hop_count,
            Timeout=timeout,
        ))

    def time_reboot(self) -> dict:
        return self._connection.get('diagnosis/time_reboot')
